#ifndef ADIDGMAXWELLGLOBAL_H_
#define ADIDGMAXWELLGLOBAL_H_

#include <deal.II/fe/fe_values.h>
#include <deal.II/fe/fe_system.h>
#include <deal.II/fe/mapping_q1.h>
#include <deal.II/fe/fe_dgq.h>

#include <deal.II/lac/sparse_matrix.h>
#include <deal.II/lac/block_sparse_matrix.h>
#include <deal.II/lac/block_vector.h>

#include "../data_types/ADIMassMatrix.h"
#include "../data_types/ADIMatrix.h"
#include "../data/Eps.h"
#include "../data/Mu.h"

#include "ADIDGMaxwellLocal.h"

using namespace dealii;

namespace MaxwellProblem {
namespace Assembling{

template<int dim>
class ADIDGMaxwellGlobal
{
public:
	ADIDGMaxwellGlobal(
			FESystem<dim>			&fe,
			const MappingQ1<dim>	&mapping,
			const QGauss<dim>		&quadrature,
			const QGauss<dim-1>		&face_quadrature,
			DoFHandler<dim>			&dof_handler
			);

	void assemble_stiffness_matrix (
			ADIMatrix<dim>	&mat
			);

	void assemble_mass_matrix (
			ADIMassMatrix<dim> & mass_matrix
			);


private:
	const Data::Mu<dim>				mu_function;
	const Data::Eps<dim>			eps_function;

	FESystem<dim> 					&fe;
	const MappingQ1<dim> 			&mapping;
	const QGauss<dim> 				&quadrature;
	const QGauss<dim-1>				&face_quadrature;
	DoFHandler<dim>					&dof_handler;
	const ADIDGMaxwellLocal<dim>	dg;

	FEValues<dim>					fe_v;
	FEFaceValues<dim>				fe_v_face;
    FESubfaceValues<dim> 			fe_v_subface;
	FEFaceValues<dim>  				fe_v_face_neighbor;
};

}
}

#endif /* ADIDGMAXWELLGLOBAL_H_ */
