#ifndef ADIDGMAXWELLLOCAL_H_
#define ADIDGMAXWELLLOCAL_H_


#include <deal.II/fe/fe_values.h>
#include <deal.II/fe/fe_system.h>
#include <deal.II/fe/mapping_q1.h>
#include <deal.II/fe/fe_dgq.h>

#include "../data/Eps.h"
#include "../data/Mu.h"

#include <deal.II/lac/sparse_matrix.h>

#include <vector>

namespace MaxwellProblem {
namespace Assembling {

using namespace dealii;

template<int dim>
class ADIDGMaxwellLocal
{
	public:
	ADIDGMaxwellLocal();

	void assemble_split_cell_curl(
			const FEValues<dim>										&fe_v,
			FullMatrix<double> 										&cell_curl_matrix,
			Vector<double>											&cell_vector,
			unsigned int											direction) const;


	void assemble_split_face_curl(
			const FEFaceValuesBase<dim>								&fe_v,
			const FEFaceValuesBase<dim>								&fe_v_neighbor,
			FullMatrix<double>										&cell_curl_matrix,
			FullMatrix<double>										&cell_face_matrix_ext_int,
			FullMatrix<double>										&cell_face_matrix_int_ext,
			FullMatrix<double>										&cell_curl_matrix_neighbor,
			unsigned int											direction
			) const;


	void assemble_split_face_curl(
			const FEFaceValuesBase<dim>								&fe_v,
			const FEFaceValuesBase<dim>								&fe_v_neighbor,
			FullMatrix<double>										&cell_curl_matrix,
			FullMatrix<double>										&cell_face_matrix_ext_int,
			FullMatrix<double>										&cell_face_matrix_int_ext,
			FullMatrix<double>										&cell_curl_matrix_neighbor,
			double													eps,
			double 													mu,
			double 													eps_neighbor,
			double													mu_neighbor,
			unsigned int											direction
			) const;

	void assemble_cell_mass_matrix(
			FullMatrix<double> 	&cell_mass_matrix,
			FEValues<dim>		&fe,
			double				eps,
			double				mu) const;


	void assemble_cell_mass_matrices(
			FullMatrix<double>	&cell_mass_matrix_H,
			FullMatrix<double> 	&cell_mass_matrix_E,
			FEValues<dim>		&fe_v,
			double				eps,
			double				mu) const;


	void assemble_cell_mass_matrices(
			FullMatrix<double>	&cell_mass_matrix_H,
			FullMatrix<double> 	&cell_mass_matrix_E,
			FEValues<dim>		&fe_v) const;

private:

	const FEValuesExtractors::Vector											E;
	const FEValuesExtractors::Vector											H;

	const Data::Mu<dim> mu_function;

	const Data::Eps<dim> eps_function;

	std::vector< std::vector< std::pair< unsigned int, unsigned int > > >	index_deriv_pairs;
	std::vector< int >														sign;
};

}
}

#endif /* ADIDGMAXWELLLOCAL_H_ */
