#ifndef DGMAXWELLGLOBAL_H_
#define DGMAXWELLGLOBAL_H_

#include <deal.II/fe/fe_values.h>
#include <deal.II/fe/fe_system.h>
#include <deal.II/fe/mapping_q1.h>
#include <deal.II/fe/fe_dgq.h>

#include <deal.II/lac/vector.h>
#include <deal.II/lac/block_vector.h>
#include <deal.II/lac/sparse_matrix.h>
#include <deal.II/lac/block_sparse_matrix.h>
//#include "../data_types/DG_MassMatrix.h"

#include <deal.II/base/function.h>
#include "../data/FunctionInterface.h"
#include "../data/Eps.h"
#include "../data/Mu.h"

#include "DGMaxwellLocal.h"

using namespace dealii;

namespace MaxwellProblem {
namespace Assembling{

template<int dim>
class DGMaxwellGlobal
{
public:
	DGMaxwellGlobal(FESystem<dim>		&fe,
			const MappingQ1<dim>		&mapping,
			const Quadrature<dim>		&quadrature,
			const Quadrature<dim-1>		&face_quadrature,
			DoFHandler<dim>				&dof_handler);

	template<class MatrixType = SparseMatrix<double> >
	void assemble_mass_matrix(MatrixType &mass_matrix_H,
			MatrixType &mass_matrix_E);

	template<class MatrixType = SparseMatrix<double> >
	void assemble_mass_matrix(MatrixType &mass_matrix_H,
						MatrixType &mass_matrix_E,
						MatrixType &mass_matrix_H_inv,
						MatrixType &mass_matrix_E_inv);

	template<class MatrixType = SparseMatrix<double> >
	void assemble_discrete_curl_matrices (MatrixType &C_H, MatrixType &C_E);

private:

	const Data::Mu<dim> mu_function;

	const Data::Eps<dim> eps_function;

	FESystem<dim> &fe;

	const MappingQ1<dim> &mapping;

	const Quadrature<dim> &quadrature;

	const Quadrature<dim-1>	&face_quadrature;

	DoFHandler<dim>	&dof_handler;

	const DGMaxwellLocal<dim> dg;

	const unsigned int dofs_per_cell;

	FEValues<dim>			fe_v;

	FEFaceValues<dim>		fe_v_face;

    FESubfaceValues<dim> 	fe_v_subface;

	FEFaceValues<dim>  		fe_v_face_neighbor;
};

} /* namespace Assembling */
} /* namespace MaxwellProblem */

#endif /* DGMAXWELLGLOBAL_H_ */
