/*
 * CavitySolution.h
 *
 *  Created on: Feb 3, 2016
 *      Author: koehler, carle
 */

#ifndef CAVITYSOLUTION_H_
#define CAVITYSOLUTION_H_


#include <deal.II/lac/vector.h>

#include "FunctionInterface.h"

using namespace dealii;

namespace MaxwellProblem {
namespace Data{

template <int dim>
class CavitySolution_H : public FunctionInterface<dim>
{
public:

	CavitySolution_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double sin_omega_t;

    const double pi;

    unsigned int	nx, ny, nz;

    double			ax,	ay,	az,
    				kx,	ky,	kz,
    				Ex, Ey,	Ez,
    				k;

    double 			mu, eps;

    double 			s_o_l;

    double			omega;
};

template <int dim>
class CavitySolution_E : public FunctionInterface<dim>
{
public:

	CavitySolution_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_omega_t;

    const double pi;

    unsigned int	nx, ny, nz;

	double			ax,	ay,	az,
    				kx,	ky,	kz,
    				Ex, Ey,	Ez;

    double 			mu, eps;

    double 			s_o_l;

	double			omega;
};

// template <int dim>
// class CavitySolutionCurl_E : public FunctionInterface<dim>
// {
// public:
// 
// 	CavitySolutionCurl_E 		(double current_time = 0.0);
// 
// 	void vector_value		(const Point<dim>				&p,
// 					 	 	 Vector<double>					&values) const;
// 
// 	void vector_value_list	(const std::vector<Point<dim> >	&points,
// 							 std::vector<Vector<double> >	&value_list) const;
// 
// 	void space_vector_value 		(const Point<dim>				&p,
// 	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;
// 
// 	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
// 									 std::vector<Vector<double>>	&value_list) const;
// 
// 	double get_time_value	(double current_time);
// 
// private:
// 
// 	double current_time;
// 
// 	double omega_sin_omega_t;
// 
//     const double pi;
// 
//     unsigned int	nx, ny, nz;
// 
//     double			ax,	ay,	az,
//     				kx,	ky,	kz,
//     				Ex, Ey,	Ez,
//     				k;
// 
//     double 			mu, eps;
// 
//     double 			s_o_l;
// 
//     double			omega;
// };
// 
// template <int dim>
// class CavitySolutionCurl_H : public FunctionInterface<dim>
// {
// public:
// 
// 	CavitySolutionCurl_H 		(double current_time = 0.0);
// 
// 	void vector_value		(const Point<dim>				&p,
// 					 	 	 Vector<double>					&values) const;
// 
// 	void vector_value_list	(const std::vector<Point<dim> >	&points,
// 							 std::vector<Vector<double> >	&value_list) const;
// 
// 	void space_vector_value 		(const Point<dim>				&p,
// 	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;
// 
// 	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
// 									 std::vector<Vector<double>>	&value_list) const;
// 
// 	double get_time_value	(double current_time);
// 
// private:
// 
// 	double current_time;
// 
// 	double cos_omega_t;
// 
//     const double pi;
// 
//     unsigned int	nx, ny, nz;
// 
// 	double			ax,	ay,	az,
//     				kx,	ky,	kz,
//     				Ex, Ey,	Ez;
// 
//     double 			mu, eps;
// 
//     double 			s_o_l;
// 
// 	double			omega;
// };

template <int dim>
class CavitySolutionDt_H : public FunctionInterface<dim>
{
public:

	CavitySolutionDt_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_omega_t;

    const double pi;

    unsigned int	nx, ny, nz;

    double			ax,	ay,	az,
    				kx,	ky,	kz,
    				Ex, Ey,	Ez,
    				k;

    double 			mu, eps;

    double 			s_o_l;

    double			omega;
};

template <int dim>
class CavitySolutionDt_E : public FunctionInterface<dim>
{
public:

	CavitySolutionDt_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double omega_sin_omega_t;

    const double pi;

    unsigned int	nx, ny, nz;

	double			ax,	ay,	az,
    				kx,	ky,	kz,
    				Ex, Ey,	Ez;

    double 			mu, eps;

    double 			s_o_l;

	double			omega;
};

} /* namespace Data */
} /* namespace MaxwellProblem */

#endif /* CAVITYSOLUTION_H_ */
