/*
 * Eps.cpp
 *
 *  Created on: Feb 2, 2016
 *      Author: koehler, leibold
 */

#include "Eps.h"

namespace MaxwellProblem {
namespace Data{

template <int dim>
Eps<dim>::Eps()
	:
	Function<dim>(dim),
	jumpingEpsMu( MaxwellProblem::global::config.get("Data.TestCase","") == "JumpSolution" ),
	lipschEpsMu( MaxwellProblem::global::config.get("Data.TestCase","") == "LipschSolution" ),
	jump_pos( MaxwellProblem::global::config.get("Data.JumpPos",.5) ),
	jump_eps( MaxwellProblem::global::config.get("Data.JumpEps",1) )
{}

template <int dim>
double Eps<dim>::value(
		const Point<dim>&	point,
		const unsigned int  /*component*/) const
{
	double eps_value(1);
	
	if ( jumpingEpsMu )
	{
		if ( point[0] > jump_pos )
			eps_value = jump_eps;
	}
	else if ( lipschEpsMu )
		eps_value = point[0]*point[0]+1;
	else
		eps_value = MaxwellProblem::global::config.get<double>("Data.Eps") ;

	return eps_value;
}

template <int dim>
void Eps<dim>::vector_value(
		const Point<dim>	&point,
		Vector<double>		&values) const
{
	// Since epsilon is isotropic, the three components are equal.
	// Therefore they can be computed from the value method.
	double tmp_value(0);
	tmp_value = value(point);

	values(0) = tmp_value;
    values(1) = tmp_value;
    values(2) = tmp_value;
}

template class Eps<3>;

} /* namespace Data */
} /* namespace MaxwellProblem */
