/*
 * EpsMu.cpp
 *
 *  Created on: Feb 2, 2016
 *      Author: koehler, leibold
 */

#include "EpsMu.h"

namespace MaxwellProblem {
namespace Data{

template <int dim>
EpsMu<dim>::EpsMu()
	:
	Function<dim>(2*dim)
{}

template <int dim>
double EpsMu<dim>::value(
		const Point<dim>&	point	/*point*/,
		const unsigned int	component /*component*/) const
{
	double epsmu_value=0;

	if (component<3)
		epsmu_value = MaxwellProblem::global::config.get<double>("Data.Eps") ;
	else
		epsmu_value = MaxwellProblem::global::config.get<double>("Data.Mu") ;

	if (epsmu_value==0)
		Assert(true, ExcInternalError());

	return epsmu_value;
}

template <int dim>
void EpsMu<dim>::vector_value(
		const Point<dim>	&point,
		Vector<double>		&values) const
{
	// Since epsilon is isotropic, the three components are equal.
	// Therefore they can be computed from the value method.
	double tmp_value(0);
	for (unsigned int component = 0; component < 6; ++component)
	{
		tmp_value = value(point,component);
		values(component) = tmp_value;
	}
}

template class EpsMu<3>;

} /* namespace Data */
} /* namespace MaxwellProblem */
