/*
 * EpsMu.h
 *
 *  Created on: Feb 2, 2016
 *      Author: koehler, leibold
 */

#ifndef EPSMU_H_
#define EPSMU_H_

#include <deal.II/base/point.h>
#include <deal.II/base/function.h>
#include <deal.II/lac/vector.h>

#include "../global.h"
#include "Eps.h"
#include "EpsMu.h"

namespace MaxwellProblem {
namespace Data{
using namespace dealii;

/**
 * \brief This class implements the electric permittivity.
 */
template <int dim>
class EpsMu : public Function<dim>
{
public:
	/**
	 * \brief Constructor.
	 */
	EpsMu	();

	/**
	 * This routine takes a point in the dim-dimensional space and
	 * writes the corresponding permittivity at that point into
	 * the value-parameter.
	 */
	double value	(const Point<dim>	&point,
						const unsigned int  component = 0) const;

	/**
	 * This routine takes a point in the dim-dimensional space and
	 * writes the corresponding permittivity at that point into
	 * the values-vector.
	 */
	void vector_value	(const Point<dim>	&point,
						 Vector<double>		&values) const;
};

} /* namespace Data */
} /* namespace MaxwellProblem */

#endif /* EPSMU_H_ */
