/*
 * GradientField.cpp
 *
 *  Created on: 29.03.2017
 *      Author: leibold
 */

#include "GradientField.h"

namespace MaxwellProblem {
namespace Data{

template<int dim>
GradientField<dim>::GradientField()
:
Function<dim>(dim) // Only for dim=3
{}

template <int dim>
void GradientField<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>	&values) const
{
    Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim)); // Error for TM,TE?
    Assert (dim >= 2, ExcNotImplemented());


    values(0) = (2*p[0] - 2) * p[1]*(p[1] - 1) * p[2]*(p[2] - 1);
    values(1) = p[0]*(p[0] - 2) * (2*p[1] - 1) * p[2]*(p[2] - 1);
    values(2) = p[0]*(p[0] - 2) * p[1]*(p[1] - 1) * (2*p[2] - 1);
}


template <int dim>
void GradientField<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
										   std::vector< Vector<double> >	&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		GradientField<dim>::vector_value (points[p], value_list[p]);
}

template class GradientField<3>;

} /* namespace Data */
} /* namespace MaxwellProblem*/


