/*
 * JumpRHS.cpp
 *
 *  Created on: Mar 26, 2017
 *      Author: leibold
 */

#include "JumpRHS.h"

namespace MaxwellProblem {
namespace Data{

template<int dim>
JumpRHSLeft<dim>::JumpRHSLeft(double current_time)
:
FunctionInterface<dim>(current_time), // Only for dim=3
current_time(current_time),
pi (3.14159265358979),
sol_E(0)
{
    sin_t = sin(2*current_time);
}

template <int dim>
void JumpRHSLeft<dim>::vector_value(const Point<dim>	&p,
								Vector<double>	&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
	auto tmp(values);
	sol_E.space_vector_value(p, tmp);
	tmp *= 2;
	values = tmp;
	
    values(0) -= pi * pi * ( cos(2*pi*p[0]) +     cos(4*pi*p[0]) ) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
	
	values *= sin_t;
}

template <int dim>
void JumpRHSLeft<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector< Vector<double> >	&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpRHSLeft<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpRHSLeft<dim>::space_vector_value(const Point<dim>	&p,
									Vector<double>	&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
	auto tmp(values);
	sol_E.space_vector_value(p, tmp);
	tmp *= 2;
	values = tmp;
	
    values(0) -= pi * pi * ( cos(2*pi*p[0]) +     cos(4*pi*p[0]) ) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
	
// 	values += tmp;
	
}

template <int dim>
void JumpRHSLeft<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector<Vector<double>>			&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpRHSLeft<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpRHSLeft<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	sin_t = sin(2*current_time);

	return sin_t;
}

template<int dim>
JumpRHSRight<dim>::JumpRHSRight(double current_time)
:
FunctionInterface<dim>(current_time), // Only for dim=3
current_time(current_time),
pi (3.14159265358979),
sol_E(0)
{
    sin_t = sin(2*current_time);
}

template <int dim>
void JumpRHSRight<dim>::vector_value(const Point<dim>	&p,
								Vector<double>	&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
	auto tmp(values);
	sol_E.space_vector_value(p, tmp);
	tmp *= 2;
	values = tmp;
	if ( MaxwellProblem::global::config.get("Test.RHS",1) == 3 )
		values *= MaxwellProblem::global::config.get("Data.JumpEps",1);
	
    values(0) -= pi * pi * ( cos(2*pi*p[0]) +     cos(4*pi*p[0]) ) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
	
	if ( MaxwellProblem::global::config.get("Test.RHS",1) == 1 )
		values *= MaxwellProblem::global::config.get("Data.JumpEps",1) * sin_t;
	else if ( MaxwellProblem::global::config.get("Test.RHS",1) == 2 )
		values *= sin_t;
}

template <int dim>
void JumpRHSRight<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector< Vector<double> >	&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpRHSRight<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpRHSRight<dim>::space_vector_value(const Point<dim>	&p,
									Vector<double>	&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
	auto tmp(values);
	sol_E.space_vector_value(p, tmp);
	tmp *= 2;
	values = tmp;
	if ( MaxwellProblem::global::config.get("Test.RHS",1) == 3 )
		values *= MaxwellProblem::global::config.get("Data.JumpEps",1);
// 	values = 0;
	
    values(0) -= pi * pi * ( cos(2*pi*p[0]) +     cos(4*pi*p[0]) ) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) += pi * pi * ( sin(2*pi*p[0]) + 2 * sin(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
	
	if ( MaxwellProblem::global::config.get("Test.RHS",1) == 1 )
		values *= MaxwellProblem::global::config.get("Data.JumpEps",1);
	
// 	values += tmp;
	
}

template <int dim>
void JumpRHSRight<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector<Vector<double>>			&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpRHSRight<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpRHSRight<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	sin_t = sin(2*current_time);

	return sin_t;
}


template<int dim>
ADIJumpRHSLeft<dim>::ADIJumpRHSLeft(double current_time)
	:
	Function<dim>(2*dim, current_time), // Only for dim=3
	current_time(current_time),
	rhs_E(current_time)
{}

template <int dim>
void ADIJumpRHSLeft<dim>::vector_value(const Point<dim>	&p,
								Vector<double>	&values) const
{
    Assert (values.size() == 2*dim, ExcDimensionMismatch (values.size(), 2*dim));
    Assert (dim >= 2, ExcNotImplemented());
	
    Vector<double> tmp_values(dim);
    rhs_E.vector_value(p, tmp_values);
    for (unsigned int ii=0; ii<dim; ++ii)
	{
    	values[ii]=0;
    	values[ii+dim]=tmp_values[ii];
	}
}

template <int dim>
void ADIJumpRHSLeft<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector< Vector<double> >	&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		ADIJumpRHSLeft<dim>::vector_value (points[p], value_list[p]);
}

// template <int dim>
// void ADIJumpRHSLeft<dim>::space_vector_value(const Point<dim>	&p,
// 									Vector<double>	&values) const
// {
// 	Assert (dim > 2, ExcNotImplemented());
// 	Assert (values.size() == 2*dim, ExcDimensionMismatch (values.size(), 2*dim));
// 
//     Vector<double> tmp_values(dim);
//     rhs_E.space_vector_value(p, tmp_values);
//     for (unsigned int ii=0; ii<dim; ++ii)
// 	{
//     	values[ii]=0;
//     	values[ii+dim]=tmp_values[ii];
// 	}
//     
// }
// 
// template <int dim>
// void ADIJumpRHSLeft<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
// 									std::vector<Vector<double>>			&value_list) const
// {
// 	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));
// 
// 	const unsigned int n_points = points.size();
// 
// 	for (unsigned int p=0; p<n_points; ++p)
// 		ADIJumpRHSLeft<dim>::space_vector_value (points[p], value_list[p]);
// }

template <int dim>
double ADIJumpRHSLeft<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;

	return rhs_E.get_time_value(current_time);
}


template<int dim>
ADIJumpRHSRight<dim>::ADIJumpRHSRight(double current_time)
	:
	Function<dim>(2*dim, current_time), // Only for dim=3
	current_time(current_time),
	rhs_E(current_time)
{}

template <int dim>
void ADIJumpRHSRight<dim>::vector_value(const Point<dim>	&p,
								Vector<double>	&values) const
{
    Assert (values.size() == 2*dim, ExcDimensionMismatch (values.size(), 2*dim));
    Assert (dim >= 2, ExcNotImplemented());
	
    Vector<double> tmp_values(dim);
    rhs_E.vector_value(p, tmp_values);
    for (unsigned int ii=0; ii<dim; ++ii)
	{
    	values[ii]=0;
    	values[ii+dim]=tmp_values[ii];
	}
}

template <int dim>
void ADIJumpRHSRight<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector< Vector<double> >	&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		ADIJumpRHSRight<dim>::vector_value (points[p], value_list[p]);
}

// template <int dim>
// void ADIJumpRHSRight<dim>::space_vector_value(const Point<dim>	&p,
// 									Vector<double>	&values) const
// {
// 	Assert (dim > 2, ExcNotImplemented());
// 	Assert (values.size() == 2*dim, ExcDimensionMismatch (values.size(), 2*dim));
// 
//     Vector<double> tmp_values(dim);
//     rhs_E.space_vector_value(p, tmp_values);
//     for (unsigned int ii=0; ii<dim; ++ii)
// 	{
//     	values[ii]=0;
//     	values[ii+dim]=tmp_values[ii];
// 	}
//     
// }
// 
// template <int dim>
// void ADIJumpRHSRight<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
// 									std::vector<Vector<double>>			&value_list) const
// {
// 	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));
// 
// 	const unsigned int n_points = points.size();
// 
// 	for (unsigned int p=0; p<n_points; ++p)
// 		ADIJumpRHSRight<dim>::space_vector_value (points[p], value_list[p]);
// }

template <int dim>
double ADIJumpRHSRight<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;

	return rhs_E.get_time_value(current_time);
}


template class JumpRHSLeft<3>;
template class JumpRHSRight<3>;
template class ADIJumpRHSLeft<3>;
template class ADIJumpRHSRight<3>;

} /* namespace Data */
} /* namespace MaxwellProblem*/
