#include "JumpSolution.h"

namespace MaxwellProblem {
namespace Data{

template <int dim>
JumpSolutionLeft_H<dim>::JumpSolutionLeft_H(double current_time)
	:
	FunctionInterface<dim>(current_time),
	current_time(current_time),
	pi (3.14159265358979)
{
    sin_t = sin(2*current_time);
}

template <int dim>
void JumpSolutionLeft_H<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

	
    values(0) = 0;
    values(1) = - .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]) * sin_t;
    values(2) =   .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]) * sin_t;
}

template <int dim>
void JumpSolutionLeft_H<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
										   std::vector<Vector<double>>			&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionLeft_H<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpSolutionLeft_H<dim>::space_vector_value(const Point<dim> &p,
											Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

	
    values(0) = 0;
    values(1) = - .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
    values(2) =   .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
}

template <int dim>
void JumpSolutionLeft_H<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
										   std::vector<Vector<double>>				&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionLeft_H<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpSolutionLeft_H<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	sin_t = sin(2*current_time);

	return sin_t;
}

template <int dim>
JumpSolutionRight_H<dim>::JumpSolutionRight_H(double current_time)
	:
	FunctionInterface<dim>(current_time),
	current_time(current_time),
	pi (3.14159265358979)
{
    sin_t = sin(2*current_time);
}

template <int dim>
void JumpSolutionRight_H<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
    values(0) = 0;
    values(1) = - MaxwellProblem::global::config.get("Data.JumpEps",1) * .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]) * sin_t;
    values(2) =   MaxwellProblem::global::config.get("Data.JumpEps",1) * .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]) * sin_t;
}

template <int dim>
void JumpSolutionRight_H<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
										   std::vector<Vector<double>>			&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionRight_H<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpSolutionRight_H<dim>::space_vector_value(const Point<dim> &p,
											Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

	
    values(0) = 0;
    values(1) = - MaxwellProblem::global::config.get("Data.JumpEps",1) * .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
    values(2) =   MaxwellProblem::global::config.get("Data.JumpEps",1) * .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
}

template <int dim>
void JumpSolutionRight_H<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
										   std::vector<Vector<double>>				&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionRight_H<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpSolutionRight_H<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	sin_t = sin(2*current_time);

	return sin_t;
}


template <int dim>
JumpSolutionLeft_E<dim>::JumpSolutionLeft_E(double current_time)
	:
	FunctionInterface<dim>(current_time),
	current_time(current_time),
	pi (3.14159265358979)
{
    cos_t = cos(2*current_time);
}

template <int dim>
void JumpSolutionLeft_E<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>		&values) const
{
    Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
    values(0) =         cos(2*pi*p[0]) * sin(pi*p[1]) *	sin(pi*p[2]) * cos_t;
    values(1) = - .25 * sin(4*pi*p[0]) * cos(pi*p[1]) *	sin(pi*p[2]) * cos_t;
    values(2) = - .25 * sin(4*pi*p[0]) * sin(pi*p[1]) *	cos(pi*p[2]) * cos_t;
}

template <int dim>
void JumpSolutionLeft_E<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
										   std::vector<Vector<double>>		&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionLeft_E<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpSolutionLeft_E<dim>::space_vector_value(const Point<dim> &p,
											Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

    values(0) =         cos(2*pi*p[0]) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) = - .25 * sin(4*pi*p[0]) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) = - .25 * sin(4*pi*p[0]) * sin(pi*p[1]) *	cos(pi*p[2]);
}

template <int dim>
void JumpSolutionLeft_E<dim>::space_vector_value_list(const std::vector< Point<dim>>	&points,
										   std::vector< Vector<double>>				&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionLeft_E<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpSolutionLeft_E<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	cos_t = cos(2*current_time);

	return cos_t;
}


template <int dim>
JumpSolutionRight_E<dim>::JumpSolutionRight_E(double current_time)
	:
	FunctionInterface<dim>(current_time),
	current_time(current_time),
	pi (3.14159265358979)
{
    cos_t = cos(2*current_time);
}

template <int dim>
void JumpSolutionRight_E<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>		&values) const
{
    Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
    values(0) =    MaxwellProblem::global::config.get("Data.JumpEps",1) * cos(2*pi*p[0]) * sin(pi*p[1]) *	sin(pi*p[2]) * cos_t;
    values(1) = - MaxwellProblem::global::config.get("Data.JumpEps",1) * .25 * sin(4*pi*p[0]) * cos(pi*p[1]) *	sin(pi*p[2]) * cos_t;
    values(2) = - MaxwellProblem::global::config.get("Data.JumpEps",1) * .25 * sin(4*pi*p[0]) * sin(pi*p[1]) *	cos(pi*p[2]) * cos_t;
}

template <int dim>
void JumpSolutionRight_E<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
										   std::vector<Vector<double>>		&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionRight_E<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpSolutionRight_E<dim>::space_vector_value(const Point<dim> &p,
											Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
    values(0) =    MaxwellProblem::global::config.get("Data.JumpEps",1) * cos(2*pi*p[0]) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) = - MaxwellProblem::global::config.get("Data.JumpEps",1) * .25 * sin(4*pi*p[0]) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) = - MaxwellProblem::global::config.get("Data.JumpEps",1) * .25 * sin(4*pi*p[0]) * sin(pi*p[1]) *	cos(pi*p[2]);
}

template <int dim>
void JumpSolutionRight_E<dim>::space_vector_value_list(const std::vector< Point<dim>>	&points,
										   std::vector< Vector<double>>				&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionRight_E<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpSolutionRight_E<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	cos_t = cos(2*current_time);

	return cos_t;
}


template <int dim>
JumpSolutionLeft<dim>::JumpSolutionLeft(double current_time)
	:
	Function<dim>(2*dim, current_time), // Only for dim=3
	current_time(current_time),
	solution_H(current_time),
	solution_E(current_time)
{}

template <int dim>
void JumpSolutionLeft<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>	&values) const
{
//     Assert (values.size() == 2*dim, ExcDimensionMismatch (values.size(), 2*dim));
    Assert (dim >= 2, ExcNotImplemented());

    Vector<double> tmp_values(dim);
    solution_H.vector_value(p, tmp_values);
    for (int i=0; i<dim; i++)
    	values[i]=tmp_values[i];
    solution_E.vector_value(p, tmp_values);
    for (int i=0; i<dim; i++)
    	values[i+dim]=tmp_values[i];
}

template <int dim>
void JumpSolutionLeft<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
										   std::vector< Vector<double> >	&value_list) const
{
// 	Assert (value_list.size() == 2*points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionLeft<dim>::vector_value (points[p], value_list[p]);
}


template <int dim>
JumpSolutionRight<dim>::JumpSolutionRight(double current_time)
	:
	Function<dim>(2*dim, current_time), // Only for dim=3
	current_time(current_time),
	solution_H(current_time),
	solution_E(current_time)
{}

template <int dim>
void JumpSolutionRight<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>	&values) const
{
//     Assert (values.size() == 2*dim, ExcDimensionMismatch (values.size(), 2*dim));
    Assert (dim >= 2, ExcNotImplemented());

    Vector<double> tmp_values(dim);
    solution_H.vector_value(p, tmp_values);
    for (int i=0; i<dim; i++)
    	values[i]=tmp_values[i];
    solution_E.vector_value(p, tmp_values);
    for (int i=0; i<dim; i++)
    	values[i+dim]=tmp_values[i];
}

template <int dim>
void JumpSolutionRight<dim>::vector_value_list(const std::vector< Point<dim> >	&points,
										   std::vector< Vector<double> >	&value_list) const
{
// 	Assert (value_list.size() == 2*points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolutionRight<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
JumpSolution_H<dim>::JumpSolution_H(double current_time)
	:
	FunctionInterface<dim>(current_time),
	current_time(current_time),
	pi (3.14159265358979)
{
    sin_t = sin(2*current_time);
}

template <int dim>
void JumpSolution_H<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

	
    values(0) = 0;
    values(1) = - .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]) * sin_t;
    values(2) =   .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]) * sin_t;
	
	if ( p[0] > .5 || std::fabs( p[0]-.5 ) < 1e-12 )
	{
		values(1) *= MaxwellProblem::global::config.get("Data.JumpEps",1);
		values(2) *= MaxwellProblem::global::config.get("Data.JumpEps",1);
	}
}

template <int dim>
void JumpSolution_H<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
										   std::vector<Vector<double>>			&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolution_H<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpSolution_H<dim>::space_vector_value(const Point<dim> &p,
											Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

	
    values(0) = 0;
    values(1) = - .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * sin(pi*p[1]) *	cos(pi*p[2]);
    values(2) =   .5 * pi *( cos(2*pi*p[0]) + cos(4*pi*p[0]) ) * cos(pi*p[1]) *	sin(pi*p[2]);
	
	if ( p[0] > .5 || std::fabs( p[0]-.5 ) < 1e-12 )
	{
		values(1) *= MaxwellProblem::global::config.get("Data.JumpEps",1);
		values(2) *= MaxwellProblem::global::config.get("Data.JumpEps",1);
	}
}

template <int dim>
void JumpSolution_H<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
										   std::vector<Vector<double>>				&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolution_H<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpSolution_H<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	sin_t = sin(2*current_time);

	return sin_t;
}


template <int dim>
JumpSolution_E<dim>::JumpSolution_E(double current_time)
	:
	FunctionInterface<dim>(current_time),
	current_time(current_time),
	pi (3.14159265358979)
{
    cos_t = cos(2*current_time);
}

template <int dim>
void JumpSolution_E<dim>::vector_value(const Point<dim>	&p,
									  Vector<double>		&values) const
{
    Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));
	
    values(0) =         cos(2*pi*p[0]) * sin(pi*p[1]) *	sin(pi*p[2]) * cos_t;
    values(1) = - .25 * sin(4*pi*p[0]) * cos(pi*p[1]) *	sin(pi*p[2]) * cos_t;
    values(2) = - .25 * sin(4*pi*p[0]) * sin(pi*p[1]) *	cos(pi*p[2]) * cos_t;
	
	if ( p[0] > .5 || std::fabs( p[0]-.5 ) < 1e-12 )
		values *= MaxwellProblem::global::config.get("Data.JumpEps",1);
}

template <int dim>
void JumpSolution_E<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
										   std::vector<Vector<double>>		&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolution_E<dim>::vector_value (points[p], value_list[p]);
}

template <int dim>
void JumpSolution_E<dim>::space_vector_value(const Point<dim> &p,
											Vector<double>		&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

    values(0) =         cos(2*pi*p[0]) * sin(pi*p[1]) *	sin(pi*p[2]);
    values(1) = - .25 * sin(4*pi*p[0]) * cos(pi*p[1]) *	sin(pi*p[2]);
    values(2) = - .25 * sin(4*pi*p[0]) * sin(pi*p[1]) *	cos(pi*p[2]);
	
	if ( p[0] > .5 || std::fabs( p[0]-.5 ) < 1e-12 )
		values *= MaxwellProblem::global::config.get("Data.JumpEps",1);
}

template <int dim>
void JumpSolution_E<dim>::space_vector_value_list(const std::vector< Point<dim>>	&points,
										   std::vector< Vector<double>>				&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		JumpSolution_E<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double JumpSolution_E<dim>::get_time_value(double current_time)
{
	this->current_time = current_time;
	cos_t = cos(2*current_time);

	return cos_t;
}


template class JumpSolution_H<3>;
template class JumpSolution_E<3>;

template class JumpSolutionLeft_H<3>;
template class JumpSolutionRight_H<3>;
template class JumpSolutionLeft_E<3>;
template class JumpSolutionRight_E<3>;
template class JumpSolutionLeft<3>;
template class JumpSolutionRight<3>;

// template class JumpSolutionDt_H<3>;
// template class JumpSolutionDt_E<3>;

} /* namespace Data */
} /* namespace MaxwellProblem */
