/*
 * JumpSolution.h
 *
 *  Created on: Feb 3, 2016
 *      Author: koehler, carle
 */

#ifndef JUMPSOLUTION_H_
#define JUMPSOLUTION_H_


#include <deal.II/lac/vector.h>

#include "FunctionInterface.h"
#include "../global.h"

using namespace dealii;

namespace MaxwellProblem {
namespace Data{

template <int dim>
class JumpSolution_H : public FunctionInterface<dim>
{
public:

	JumpSolution_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double sin_t;

    const double pi;
};

template <int dim>
class JumpSolution_E : public FunctionInterface<dim>
{
public:

	JumpSolution_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_t;

    const double pi;
};

template <int dim>
class JumpSolutionLeft_H : public FunctionInterface<dim>
{
public:

	JumpSolutionLeft_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double sin_t;

    const double pi;
};

template <int dim>
class JumpSolutionLeft_E : public FunctionInterface<dim>
{
public:

	JumpSolutionLeft_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_t;

    const double pi;
};

template <int dim>
class JumpSolutionRight_H : public FunctionInterface<dim>
{
public:

	JumpSolutionRight_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double sin_t;

    const double pi;
};

template <int dim>
class JumpSolutionRight_E : public FunctionInterface<dim>
{
public:

	JumpSolutionRight_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_t;

    const double pi;
};


template <int dim>
class JumpSolutionLeft : public Function<dim>
{
public:
	/**
	 * \brief Constructor.
	 */
	JumpSolutionLeft 			(double current_time = 0.0);
	/**
	 *  \brief Evaluates the function at a single point.
	 *
	 * 	This routine takes a point in the dim-dimensional space and
	 * 	writes the corresponding function-values at that point into
	 * 	the values-parameter.
	 */
	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;
	/**
	 * \brief Evaluates the space-function at multiple points.
	 *
	 * 	This routine takes a point in the dim-dimensional space and
	 * 	writes the corresponding function-values at that point into
	 * 	the values-parameter.
	 */
	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

private:
	/**
	 * variable of time
	 */
	double current_time;
	/**
	 * H-field of the exact solution
	 */
	JumpSolutionLeft_H<dim> solution_H;
	/**
	 * E-field of the exact solution
	 */
	JumpSolutionLeft_E<dim> solution_E;
};


template <int dim>
class JumpSolutionRight : public Function<dim>
{
public:
	/**
	 * \brief Constructor.
	 */
	JumpSolutionRight 			(double current_time = 0.0);
	/**
	 *  \brief Evaluates the function at a single point.
	 *
	 * 	This routine takes a point in the dim-dimensional space and
	 * 	writes the corresponding function-values at that point into
	 * 	the values-parameter.
	 */
	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;
	/**
	 * \brief Evaluates the space-function at multiple points.
	 *
	 * 	This routine takes a point in the dim-dimensional space and
	 * 	writes the corresponding function-values at that point into
	 * 	the values-parameter.
	 */
	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

private:
	/**
	 * variable of time
	 */
	double current_time;
	/**
	 * H-field of the exact solution
	 */
	JumpSolutionRight_H<dim> solution_H;
	/**
	 * E-field of the exact solution
	 */
	JumpSolutionRight_E<dim> solution_E;
};

// template <int dim>
// class JumpSolutionCurl_E : public FunctionInterface<dim>
// {
// public:
// 
// 	JumpSolutionCurl_E 		(double current_time = 0.0);
// 
// 	void vector_value		(const Point<dim>				&p,
// 					 	 	 Vector<double>					&values) const;
// 
// 	void vector_value_list	(const std::vector<Point<dim> >	&points,
// 							 std::vector<Vector<double> >	&value_list) const;
// 
// 	void space_vector_value 		(const Point<dim>				&p,
// 	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;
// 
// 	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
// 									 std::vector<Vector<double>>	&value_list) const;
// 
// 	double get_time_value	(double current_time);
// 
// private:
// 
// 	double current_time;
// 
// 	double omega_sin_omega_t;
// 
//     const double pi;
// 
//     unsigned int	nx, ny, nz;
// 
//     double			ax,	ay,	az,
//     				kx,	ky,	kz,
//     				Ex, Ey,	Ez,
//     				k;
// 
//     double 			mu, eps;
// 
//     double 			s_o_l;
// 
//     double			omega;
// };
// 
// template <int dim>
// class JumpSolutionCurl_H : public FunctionInterface<dim>
// {
// public:
// 
// 	JumpSolutionCurl_H 		(double current_time = 0.0);
// 
// 	void vector_value		(const Point<dim>				&p,
// 					 	 	 Vector<double>					&values) const;
// 
// 	void vector_value_list	(const std::vector<Point<dim> >	&points,
// 							 std::vector<Vector<double> >	&value_list) const;
// 
// 	void space_vector_value 		(const Point<dim>				&p,
// 	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;
// 
// 	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
// 									 std::vector<Vector<double>>	&value_list) const;
// 
// 	double get_time_value	(double current_time);
// 
// private:
// 
// 	double current_time;
// 
// 	double cos_omega_t;
// 
//     const double pi;
// 
//     unsigned int	nx, ny, nz;
// 
// 	double			ax,	ay,	az,
//     				kx,	ky,	kz,
//     				Ex, Ey,	Ez;
// 
//     double 			mu, eps;
// 
//     double 			s_o_l;
// 
// 	double			omega;
// };
/*
template <int dim>
class JumpSolutionDt_H : public FunctionInterface<dim>
{
public:

	JumpSolutionDt_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_omega_t;

    const double pi;

    unsigned int	nx, ny, nz;

    double			ax,	ay,	az,
    				kx,	ky,	kz,
    				Ex, Ey,	Ez,
    				k;

    double 			mu, eps;

    double 			s_o_l;

    double			omega;
};

template <int dim>
class JumpSolutionDt_E : public FunctionInterface<dim>
{
public:

	JumpSolutionDt_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double omega_sin_omega_t;

    const double pi;

    unsigned int	nx, ny, nz;

	double			ax,	ay,	az,
    				kx,	ky,	kz,
    				Ex, Ey,	Ez;

    double 			mu, eps;

    double 			s_o_l;

	double			omega;
};*/

} /* namespace Data */
} /* namespace MaxwellProblem */

#endif /* CAVITYSOLUTION_H_ */
