/*
 * LipschSolution.h
 *
 *  Created on: Feb 3, 2016
 *      Author: koehler, carle
 */

#ifndef LIPSCHSOLUTION_H_
#define LIPSCHSOLUTION_H_


#include <deal.II/lac/vector.h>

#include "FunctionInterface.h"

#include "Eps.h"
#include "Mu.h"

using namespace dealii;

namespace MaxwellProblem {
namespace Data{

template <int dim>
class LipschSolution_H : public FunctionInterface<dim>
{
public:

	LipschSolution_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double sin_omega_t;

    const double pi;

    double 			s_o_l;

    double			omega;
};

template <int dim>
class LipschSolution_E : public FunctionInterface<dim>
{
public:

	LipschSolution_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_omega_t;

    const double pi;

    const Eps<dim> 	eps_function;
};


template <int dim>
class LipschSolutionDt_H : public FunctionInterface<dim>
{
public:

	LipschSolutionDt_H 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double cos_omega_t;

    const double pi;

    double 			s_o_l;

    double			omega;
};

template <int dim>
class LipschSolutionDt_E : public FunctionInterface<dim>
{
public:

	LipschSolutionDt_E 		(double current_time = 0.0);

	void vector_value		(const Point<dim>				&p,
					 	 	 Vector<double>					&values) const;

	void vector_value_list	(const std::vector<Point<dim> >	&points,
							 std::vector<Vector<double> >	&value_list) const;

	void space_vector_value 		(const Point<dim>				&p,
	 	 	 	 	 	 	 	 	 Vector<double>					&values) const;

	void space_vector_value_list	(const std::vector<Point<dim>>	&points,
									 std::vector<Vector<double>>	&value_list) const;

	double get_time_value	(double current_time);

private:

	double current_time;

	double sin_omega_t;

    const double pi;

    const Eps<dim> 	eps_function;
};

} /* namespace Data */
} /* namespace MaxwellProblem */

#endif /* CAVITYSOLUTION_H_ */
