/*
 * Mu.cpp
 *
 *  Created on: Feb 2, 2016
 *      Author: koehler, leibold
 */

#include "Mu.h"

namespace MaxwellProblem {
namespace Data{

template <int dim>
Mu<dim>::Mu()
	:
	Function<dim>(dim),
	jumpingEpsMu( MaxwellProblem::global::config.get("Data.TestCase","") == "JumpSolution" ),
	lipschEpsMu( MaxwellProblem::global::config.get("Data.TestCase","") == "LipschSolution" )//,
// 	jump_pos(MaxwellProblem::global::config.get("Data.JumpPos",0.5)),
// 	mat_jump(MaxwellProblem::global::config.get("Data.JumpMu",1))
{}

template <int dim>
double Mu<dim>::value(
		const Point<dim>&	point,
		const unsigned int  /*component*/) const
{
	double mu_value(1);
	
	if (lipschEpsMu)
	{
		mu_value = point[0]*point[0]+1;
		mu_value = 1./mu_value;
	}
	else if ( !jumpingEpsMu )
		mu_value = MaxwellProblem::global::config.get<double>("Data.Mu");

	return mu_value;
}

template <int dim>
void Mu<dim>::vector_value(
		const Point<dim>	&point,
		Vector<double>		&values) const
{
	// Since mu is isotropic, the three components are equal.
	// Therefore they can be computed from the value method.
	double tmp_value(0);
	tmp_value = value(point);

	values(0) = tmp_value;
    values(1) = tmp_value;
    values(2) = tmp_value;
}

template class Mu<3>;

} /* namespace Data */
} /* namespace MaxwellProblem */
