/*
 * Mu.h
 *
 *  Created on: Feb 2, 2016
 *      Author: koehler, leibold
 */

#ifndef MU_H_
#define MU_H_

#include <deal.II/base/point.h>
#include <deal.II/base/function.h>
#include <deal.II/lac/vector.h>

#include "../global.h"

namespace MaxwellProblem {
namespace Data{
using namespace dealii;

/**
 * \brief This class implements the electric permeability..
 */
template <int dim>
class Mu : public Function<dim>
{
public:
	/**
	 * \brief Constructor.
	 */
	Mu ();

	/**
	 * This routine takes a point in the dim-dimensional space and
	 * writes the corresponding permeability at that point into
	 * the value-parameter.
	 */
	double value	(const Point<dim>	&point,
						const unsigned int  component = 0) const;

	/**
	 * This routine takes a point in the dim-dimensional space and
	 * writes the corresponding permeability at that point into
	 * the values-vector.
	 */
	void vector_value		(const Point<dim>	&point,
					 	 	 Vector<double>		&values) const;

private:
// 	const double jump_pos, jump_mu;
	const bool jumpingEpsMu, lipschEpsMu;
};

} /* namespace Data */
} /* namespace MaxwellProblem */

#endif /* MU_H_ */
