/*
 * RHS.cpp
 *
 *  Created on: Feb 2, 2016
 *      Author: koehler, carle, leibold
 */

#include "RHS.h"

namespace MaxwellProblem {
namespace Data{

template <int dim>
RHS<dim>::RHS(double time)
	:
	FunctionInterface<dim>(time),
	rhs_func_ts_sep(nullptr),
	rhs_func(nullptr),
	time(time)
{
	if (MaxwellProblem::global::config.get<std::string>("Data.TestCase") == "PolSolution")
		rhs_func_ts_sep = new PolRHS<dim>; //rhs = new PolRHS<dim>(current_time);
	else
		rhs_func_ts_sep = NULL;
}

template <int dim>
void RHS<dim>::vector_value(const Point<dim>&	p,
							Vector<double>		&values) const
{
	Assert (dim == 3, ExcNotImplemented());
	if(rhs_func == NULL)
		values = 0;
	else
		rhs_func->vector_value(p, values);
}

template <int dim>
void RHS<dim>::vector_value_list(const std::vector<Point<dim>>	&points,
								 std::vector<Vector<double>>	&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));
	if(rhs_func == NULL)
	{
		const unsigned int n_points = points.size();

		for (unsigned int p=0; p<n_points; ++p)
			value_list[p] = 0;
	}
	else
		rhs_func->vector_value_list(points, value_list);
}

template <int dim>
void RHS<dim>::space_vector_value(const Point<dim>	&p,
									Vector<double>	&values) const
{
	Assert (dim > 2, ExcNotImplemented());
	Assert (values.size() == dim, ExcDimensionMismatch (values.size(), dim));

		rhs_func_ts_sep->space_vector_value(p, values);
}

template <int dim>
void RHS<dim>::space_vector_value_list(const std::vector< Point<dim> >	&points,
									std::vector<Vector<double>>			&value_list) const
{
	Assert (value_list.size() == points.size(), ExcDimensionMismatch (value_list.size(), points.size()));

	const unsigned int n_points = points.size();

	for (unsigned int p=0; p<n_points; ++p)
		RHS<dim>::space_vector_value (points[p], value_list[p]);
}

template <int dim>
double RHS<dim>::get_time_value(double current_time)
{
	return rhs_func_ts_sep->get_time_value(current_time);
}


template class RHS<3>;

} /* namespace Data */
} /* namespace MaxwellProblem */
