/*
 * ADIBlockTridiag.h
 *
 *  Created on: Feb 16, 2016
 *      Author: koehler
 */

#ifndef ADIBLOCKTRIDIAG_H_
#define ADIBLOCKTRIDIAG_H_

#include <iostream>
#include <fstream>

#include <thread>
#include <mutex>
#include <vector>

#include <deal.II/numerics/vector_tools.h>
#include <deal.II/numerics/matrix_tools.h>
#include <deal.II/base/multithread_info.h>

#include "ADIMassMatrix.h"
#include "../reordering/ADIGeometryInfo.h"
#include "../global.h"

namespace MaxwellProblem {

using namespace dealii;

template <int dim>
class ADIBlockTridiag
{
public:
								ADIBlockTridiag				();
								ADIBlockTridiag				(const ADIBlockTridiag< dim >					&mat);
	void						initialize	 				(const ADIGeometryInfo< dim >					&geometry_info,
															 const unsigned int 							n_elements);

	void						vmult 						(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in) const;

	void						inverse_vmult				(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in) const;

//	void						mmult 						(ADIBlockTridiag< dim > 							&C,
//															 const ADIBlockTridiag< dim > 					&B) const;
//	void						Tmmult 						(ADIBlockTridiag< dim > 							&C,
//															 const ADIBlockTridiag< dim > 					&B) const;

	void						add							(const std::vector< types::global_dof_index >	&dofs,
												 	 		 const std::vector< types::global_dof_index >	&dofs_neighbor,
												 	 		 const FullMatrix< double >						&mat);
	void						add							(const types::global_dof_index					dof_offset_i,
												 	 		 const types::global_dof_index					dof_offset_j,
												 	 		 const FullMatrix< double >						&mat);
	void						add							(const types::global_dof_index					dof_offset,
												 	 		 const FullMatrix< double >						&mat);
	void						add							(const std::vector< types::global_dof_index >	&dofs,
															 const FullMatrix< double >						&mat);
	void						add							(const types::global_dof_index					dof_i,
												 	 		 const types::global_dof_index					dof_j,
												 	 		 const double									val);
	void						add							(const ADIBlockTridiag< dim >					&mat);
	void 						add							(const double 									coeff,
											 	 	 		 const ADIMassMatrix< dim >						&mat,
											 	 	 		 const std::vector< unsigned int >				&transform);
	void 						add							(const double 									coeff,
											 	 	 	 	 const ADIMassMatrix< dim >						&mat,
											 	 	 	 	 const std::vector< unsigned int >				&transform,
											 	 	 		 Timer 											&timer);
	void 						add							(const double 									coeff,
											 	 	 		 const ADIBlockTridiag< dim >					&mat);

	void 						copy_from					(const ADIBlockTridiag< dim >					&mat);

	void						invert						();

//	bool 						operator==					(const ADIBlockTridiag< dim >					&mat);
	ADIBlockTridiag<dim> &		operator*=					(const double									coeff);

	void						save_matrix					(std::ofstream									&out) const;
	void						load_matrix					(std::ifstream									&in);

	void						output_matrix				(std::stringstream								&filename) const;
	void						output_matrix_spy			(std::stringstream								&filename) const;
	void						output_matrix_matlab		(std::stringstream								&filename) const;

	unsigned int 				n							() const;
	unsigned int 				m							() const;

	void						print_norms					() const;

private:
	void 						dofs_to_matrix_dofs 		(std::vector< types::global_dof_index >			&dofs);
	void 						dofs_to_component	 		(std::vector< types::global_dof_index >			&dofs);
//	unsigned int 				dof_to_element				(const unsigned int								dof);
	unsigned int 				dof_to_matrix				(const unsigned int								dof);
	unsigned int				dof_to_matrix_dof			(const unsigned int								dof);
	unsigned int 				dof_to_component			(const unsigned int								dof);
//	void		 				global_to_local_dof			(unsigned int									&component,
//			  	  	  	  	  	  	  	  	  	  	  	  	 unsigned int									&matrix,
//			  	  	  	  	  	  	  	  	  	  	  	  	 unsigned int									&matrix_dof,
//			  	  	  	  	  	  	  	  	  	  	  	  	 const unsigned int								dof);
	void						threaded_vmult 				(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in) const;
//	void						threaded_inverse_vmult		(Vector< double > 								&vec_out,
//												 	 		 const Vector< double > 						&vec_in) const;
//	void						threaded_invert				();
	void						sequential_vmult 			(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in) const;
//	void						sequential_inverse_vmult	(Vector< double > 								&vec_out,
//												 	 		 const Vector< double > 						&vec_in) const;
//	void						sequential_invert			();
//	void						inverse_vmult_function 		(Vector< double > 								&vec_out,
//												 	 		 const Vector< double > 						&vec_in) const;
	void						vmult_function 				(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in) const;
//	void						invert_function				();

private:
	unsigned int 													dofs_per_element,
																	n_elements,
																	dim_n;

	std::vector< FullMatrix< double > >								diagonal,
																	subdiagonal,
																	superdiagonal;

	const unsigned int												n_threads;

	mutable std::stack< unsigned int >								tasks;
	mutable std::mutex												mtx;
};

} /* namespace MaxwellProblem */

#endif /* ADIBLOCKTRIDIAG_H_ */
