#ifndef ADIMATRIX_H_
#define ADIMATRIX_H_

#include <iostream>
#include <fstream>
#include <string.h>

#include <thread>
#include <mutex>
#include <vector>

#include <limits>

#include <deal.II/numerics/vector_tools.h>
#include <deal.II/numerics/matrix_tools.h>
#include <deal.II/base/multithread_info.h>

#include "ADIBlockTridiag.h"
#include "ADIMassMatrix.h"
#include "../reordering/ADIGeometryInfo.h"
#include "../global.h"

namespace MaxwellProblem {

using namespace dealii;

template <int dim>
class ADIMatrix
{
public:
								ADIMatrix				();
	void						initialize	 				(const ADIGeometryInfo< dim >					&geometry_info,
														 	 const unsigned int 							dir,
														 	 const unsigned int 							system_dir);
	void						set_reordering_flag_from_system_dir			(const unsigned int 							system_dir);
	void						set_reordering_flag			(const bool 									reord_flag);
	void						set_direction				(const unsigned int 							dir);

	void						vmult 						(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;

	void						inverse_vmult				(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;

	void						mmult 						(ADIMatrix< dim > 							&C,
															 const ADIMatrix< dim > 					&B) const;
	void						Tmmult 						(ADIMatrix< dim > 							&C,
															 const ADIMatrix< dim > 					&B) const;

	void						add							(const std::vector< types::global_dof_index >	&dofs,
												 	 		 const std::vector< types::global_dof_index >	&dofs_neighbor,
												 	 		 const FullMatrix< double >						&mat);
	void						add							(const std::vector< types::global_dof_index >	&dofs,
												 	 		 const std::vector< types::global_dof_index >	&dofs_neighbor,
												 	 		 const FullMatrix< double >						&mat,
												 	 		 Timer 											&timer);
	void						add							(const std::vector< types::global_dof_index >	&dofs,
															 const FullMatrix< double >						&mat);
	void						add							(const std::vector< types::global_dof_index >	&dofs,
															 const FullMatrix< double >						&mat,
															 Timer 											&timer);
	void						add							(const types::global_dof_index					dof_i,
												 	 		 const types::global_dof_index					dof_j,
												 	 		 const double									val);
	void						add							(const ADIMassMatrix< dim >						&mat,
												 	 		 const std::vector< unsigned int >				&transform);
	void						add							(const ADIMassMatrix< dim >						&mat,
												 	 		 const std::vector< unsigned int >				&transform,
												 	 		 Timer 											&timer);
	void						add							(const ADIMatrix< dim >					&mat);
	void 						add							(const double 									coeff,
											 	 	 		 const ADIMassMatrix< dim >						&mat,
											 	 	 		 const std::vector< unsigned int >				&transform);
	void 						add							(const double 									coeff,
											 	 	 	 	 const ADIMassMatrix< dim >						&mat,
											 	 	 	 	 const std::vector< unsigned int >				&transform,
											 	 	 		 Timer 											&timer);
	void 						add							(const double 									coeff,
											 	 	 		 const ADIMatrix< dim >					&mat);

	void 						copy_from					(const double 									coeff,
	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 const ADIMatrix< dim >					&mat);
	void 						copy_from					(const ADIMatrix< dim >					&mat);

	void						invert						();

	void						solve						(Vector< double >								&solution,
												 	 		 const Vector< double >							&rhs,
												 	 	 	 const std::vector< unsigned int >				&transform) const;

	bool 						operator==					(const ADIMatrix< dim >					&mat);
	ADIMatrix<dim> &			operator*=					(const double									coeff);

	void						output_matrix				(std::stringstream								&filename) const;
	void						output_matrix_spy			(std::stringstream								&filename) const;
	void						output_matrix_matlab		(std::stringstream								&filename) const;
	
	void						save_matrix					(std::stringstream								&filename) const;
	void						load_matrix					(std::stringstream								&filename);

	unsigned int 				get_direction				() const;
	unsigned int 				n							() const;
	unsigned int 				m							() const;
	const SparseMatrix<double>&	get_matrix					(unsigned int i) const;

private:
	void 						dofs_to_matrix_dofs 		(std::vector< types::global_dof_index >			&dofs);
	void 						dofs_to_component	 		(std::vector< types::global_dof_index >			&dofs);
//	unsigned int 				dof_to_element				(const unsigned int								dof);
	unsigned int 				dof_to_matrix				(const unsigned int								dof);
	unsigned int				dof_to_matrix_dof			(const unsigned int								dof);
	unsigned int 				dof_to_component			(const unsigned int								dof);
//	void		 				global_to_local_dof			(unsigned int									&component,
//			  	  	  	  	  	  	  	  	  	  	  	  	 unsigned int									&matrix,
//			  	  	  	  	  	  	  	  	  	  	  	  	 unsigned int									&matrix_dof,
//			  	  	  	  	  	  	  	  	  	  	  	  	 const unsigned int								dof);
	unsigned int				transform_dofs				(const unsigned int								dof,
														 	 const std::vector< unsigned int >				&transform) const;
	void						transform_dofs				(std::vector< unsigned int >					&transformed_dofs,
														 	 const std::vector< unsigned int >				&dofs,
															 const std::vector< unsigned int >				&transform);
	void						threaded_solve				(Vector< double >								&solution,
												 	 		 const Vector< double >							&rhs,
												 	 	 	 const std::vector< unsigned int >				&transform) const;
	void						threaded_vmult 				(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;
	void						threaded_inverse_vmult		(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;
	void						threaded_invert				();
	void						sequential_solve			(Vector< double >								&solution,
												 	 		 const Vector< double >							&rhs,
												 	 	 	 const std::vector< unsigned int >				&transform) const;
	void						sequential_vmult 			(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;
	void						sequential_inverse_vmult	(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;
	void						sequential_invert			();
	void						solve_function				(Vector< double >								&solution,
												 	 		 const Vector< double >							&rhs,
												 	 	 	 const std::vector< unsigned int >				&transform) const;
	void						inverse_vmult_function 		(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;
	void						vmult_function 				(Vector< double > 								&vec_out,
												 	 		 const Vector< double > 						&vec_in,
												 	 		 const std::vector< unsigned int > 				&transform) const;
	void						invert_function				();

private:
	unsigned int 													direction,
																	comp_dofs_per_cell,
																	dim_n,
																	dim_m;

	bool 															reordering;

	std::vector< std::vector< ADIBlockTridiag< dim > > >			matrices;
	std::vector< std::vector< unsigned int > > 						matrix_sizes, offset;
	std::vector< unsigned int > 									dof2component, n_dofs_per_component, dof2matrix, dof2matrixdof;

	const unsigned int												n_threads;

	mutable std::stack< std::pair< unsigned int, unsigned int > >	tasks;
	mutable std::mutex												mtx;
};

} /* namespace MaxwellProblem */

#endif /* ADIMATRIX_H_ */
